package com.viafirma.fortress.demo.controller;

import com.viafirma.fortress.demo.configuration.FortressDemoConfiguration;
import com.viafirma.fortress.demo.exception.FortressDemoException;
import com.viafirma.fortress.demo.service.FortressDemoService;
import com.viafirma.fortress.sdk.model.AccessToken;
import com.viafirma.fortress.sdk.model.UserStatus;
import com.viafirma.fortress.sdk.model.signature.SignatureRequestResponse;
import com.viafirma.fortress.sdk.model.signature.SignatureResponse;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.util.List;

import static org.springframework.http.HttpStatus.OK;
import static org.springframework.web.bind.annotation.RequestMethod.GET;

@Controller
public class FortressDemoController {

    private static final Logger LOGGER = LoggerFactory.getLogger(FortressDemoController.class);
    private static final String REDIRECT_PREFFIX = "redirect:";
    private static final String OPTIONS_VIEW = "options";
    private static final String CANCEL_VIEW = "cancel";
    private static final String SIGNATURE_VIEW = "signature";
    private static final String EXE_CODE_ATTRIBUTE = "exeCode";

    private final FortressDemoService fortressDemoService;
    private final FortressDemoConfiguration fortressDemoConfiguration;

    @Autowired
    public FortressDemoController(FortressDemoService fortressDemoService, FortressDemoConfiguration fortressDemoConfiguration) {
        this.fortressDemoService = fortressDemoService;
        this.fortressDemoConfiguration = fortressDemoConfiguration;
    }

    @RequestMapping(value = "/", method = GET)
    public ModelAndView index(HttpServletRequest request, Principal principal) throws FortressDemoException {

        ModelAndView modelAndView = new ModelAndView(OPTIONS_VIEW);

        try {
            String userCode = principal.getName();
            UserStatus userStatus = fortressDemoService.getUserStatus(userCode);

            modelAndView.addObject("userStatus", userStatus);
        } catch (FortressDemoException e) {
            LOGGER.error("Ha ocurrido un error.", e);
        }
        return modelAndView;
    }

    @RequestMapping(value = "/auth", method = GET)
    public String auth(Principal principal) throws FortressDemoException {
        return REDIRECT_PREFFIX + fortressDemoService.getAuthUri(String.format("%sauth/response", fortressDemoConfiguration.getBaseUrl()), principal.getName(), "profile", null);
    }

    @RequestMapping(value = "/auth/response", method = GET)
    public ModelAndView authResponse(@RequestParam(value = "code", required = false) String code, @RequestParam(value = "error", required = false) String error,
            @RequestParam(value = "state", required = false) String state) throws FortressDemoException {

        if (error != null) {
            return new ModelAndView(CANCEL_VIEW);
        } else {
            AccessToken accessToken = fortressDemoService.getAccessToken(code, fortressDemoConfiguration.getBaseUrl() + "auth/response");

            ModelAndView modelAndView = new ModelAndView("authentication");
            modelAndView.addObject("userCode", accessToken.getUserCode());
            return modelAndView;
        }
    }

    @RequestMapping(value = "/sign/{format}", method = GET)
    public String prepareSign(@PathVariable("format") String format, Principal principal, HttpServletRequest request) throws FortressDemoException {
        SignatureRequestResponse signatureRequestResponse;
        if (FortressDemoService.PDF_FORMAT.equals(format)) {
            signatureRequestResponse = fortressDemoService.prepareSignPdf(principal.getName());
        } else if (FortressDemoService.XML_FORMAT.equals(format)) {
            signatureRequestResponse = fortressDemoService.prepareSignXml(principal.getName());
        } else {
            signatureRequestResponse = fortressDemoService.prepareSignBatch(principal.getName());
        }
        String authCode = signatureRequestResponse.getAuthCode();
        String exeCode = signatureRequestResponse.getExeCode();
        request.getSession().setAttribute(EXE_CODE_ATTRIBUTE, exeCode);
        return REDIRECT_PREFFIX + fortressDemoService.getAuthSignatureUri(authCode);
    }

    @RequestMapping(value = "/sign/response", method = GET)
    public ModelAndView sign(@RequestParam(value = "error", required = false) String error, HttpServletRequest request) throws FortressDemoException {
        String executionCode = (String) request.getSession().getAttribute(EXE_CODE_ATTRIBUTE);

        if (error != null || executionCode == null) {
            return new ModelAndView(CANCEL_VIEW);
        } else {
            List<SignatureResponse> signatureResponses = fortressDemoService.executeSignature(executionCode);

            ModelAndView modelAndView = new ModelAndView(SIGNATURE_VIEW);
            modelAndView.addObject("signatures", signatureResponses);

            return modelAndView;
        }
    }

    @RequestMapping(value = "/document/{format}/{signatureCode}", method = GET)
    public ResponseEntity<byte[]> getDocument(@PathVariable("format") String format, @PathVariable("signatureCode") String signatureCode) throws FortressDemoException {

        byte[] document = fortressDemoService.getDocument(signatureCode);

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.parseMediaType("application/" + format));
        headers.setContentDispositionFormData(signatureCode, signatureCode + "." + format);
        headers.setCacheControl("must-revalidate, post-check=0, pre-check=0");
        return new ResponseEntity<>(document, headers, OK);
    }

    @ExceptionHandler(FortressDemoException.class)
    public ModelAndView fortressDemoExceptionHandler(FortressDemoException e) {
        LOGGER.error(e.getMessage(), e);

        ModelAndView modelAndView = new ModelAndView("exception");
        modelAndView.addObject("exception", e);
        return modelAndView;
    }
}
