package com.viafirma.fortress.demo.service;

import com.viafirma.fortress.demo.configuration.FortressDemoConfiguration;
import com.viafirma.fortress.demo.exception.FortressDemoException;
import com.viafirma.fortress.sdk.FortressApi;
import com.viafirma.fortress.sdk.exception.ApiException;
import com.viafirma.fortress.sdk.model.AccessToken;
import com.viafirma.fortress.sdk.model.UserStatus;
import com.viafirma.fortress.sdk.model.signature.Document;
import com.viafirma.fortress.sdk.model.signature.PadesConfiguration;
import com.viafirma.fortress.sdk.model.signature.SignatureConfiguration;
import com.viafirma.fortress.sdk.model.signature.SignatureConfiguration.Packaging;
import com.viafirma.fortress.sdk.model.signature.SignatureConfiguration.SignatureAlgorithm;
import com.viafirma.fortress.sdk.model.signature.SignatureConfiguration.SignatureType;
import com.viafirma.fortress.sdk.model.signature.SignatureRequest;
import com.viafirma.fortress.sdk.model.signature.SignatureRequestResponse;
import com.viafirma.fortress.sdk.model.signature.SignatureResponse;
import com.viafirma.fortress.sdk.model.signature.Stamper;
import com.viafirma.fortress.sdk.model.signature.Stamper.Type;
import com.viafirma.fortress.sdk.model.signature.XadesConfiguration;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import static com.viafirma.fortress.sdk.FortressApi.GRANT_TYPE_AUTHORIZATION_CODE;
import static com.viafirma.fortress.sdk.FortressApi.GRANT_TYPE_CLIENT_CREDENTIALS;

@Service
public class FortressDemoService {

    private final FortressApi fortressApi;
    private final FortressDemoConfiguration fortressDemoConfiguration;

    private final String logoB64;
    private final String pdfB64;
    private final String xmlB64;

    private static final String PDF_NAME = "contract.pdf";
    private static final String XML_NAME = "invoice.xml";
    private static final String LOGO_NAME = "logo.png";

    public static final String PDF_FORMAT = "pdf";
    public static final String XML_FORMAT = "xml";

    private final String redirectUri;

    @Autowired
    public FortressDemoService(FortressDemoConfiguration fortressDemoConfiguration) throws IOException {
        this.fortressDemoConfiguration = fortressDemoConfiguration;
        fortressApi = new FortressApi(fortressDemoConfiguration.getApi());

        redirectUri = fortressDemoConfiguration.getBaseUrl() + "sign/response";

        try (InputStream logoInputStream = FortressDemoService.class.getResourceAsStream("/" + LOGO_NAME);
                InputStream pdfInputStream = FortressDemoService.class.getResourceAsStream("/" + PDF_NAME);
                InputStream xmlInputStream = FortressDemoService.class.getResourceAsStream("/" + XML_NAME)) {

            logoB64 = Base64.encodeBase64String(IOUtils.toByteArray(logoInputStream));
            pdfB64 = Base64.encodeBase64String(IOUtils.toByteArray(pdfInputStream));
            xmlB64 = Base64.encodeBase64String(IOUtils.toByteArray(xmlInputStream));
        }
    }

    public UserStatus getUserStatus(String userCode) throws FortressDemoException {
        try {
            AccessToken accessToken = getClientAccessToken();
            return fortressApi.getUserStatus(accessToken.getAccessToken(), userCode);
        } catch (ApiException e) {
            throw new FortressDemoException(e.getMessage(), e);
        }
    }

    public SignatureRequestResponse prepareSignPdf(String userCode) throws FortressDemoException {
        try {
            AccessToken accessToken = getClientAccessToken();
            return fortressApi.signatureRequest(accessToken.getAccessToken(), getPadesSignatureRequest(userCode));
        } catch (ApiException e) {
            throw new FortressDemoException(e.getMessage(), e);
        }
    }

    public SignatureRequestResponse prepareSignXml(String userCode) throws FortressDemoException {
        try {
            AccessToken accessToken = getClientAccessToken();
            return fortressApi.signatureRequest(accessToken.getAccessToken(), getXadesSignatureRequest(userCode));
        } catch (ApiException e) {
            throw new FortressDemoException(e.getMessage(), e);
        }
    }

    public SignatureRequestResponse prepareSignBatch(String userCode) throws FortressDemoException {
        try {
            AccessToken accessToken = getClientAccessToken();
            return fortressApi.signatureRequest(accessToken.getAccessToken(), geBatchSignatureRequest(userCode));
        } catch (ApiException e) {
            throw new FortressDemoException(e.getMessage(), e);
        }
    }

    public List<SignatureResponse> executeSignature(String executionCode) throws FortressDemoException {
        try {
            AccessToken accessToken = getClientAccessToken();
            return fortressApi.executeSignature(accessToken.getAccessToken(), executionCode);
        } catch (ApiException e) {
            throw new FortressDemoException(e.getMessage(), e);
        }
    }

    private SignatureRequest getPadesSignatureRequest(String userCode) {
        ArrayList<SignatureConfiguration> signatureConfigurations = new ArrayList<>();
        signatureConfigurations.add(getPadesSignatureConfiguration());

        SignatureRequest signatureRequest = new SignatureRequest();
        signatureRequest.setSignatureConfigurations(signatureConfigurations);
        signatureRequest.setUserCode(userCode);

        return signatureRequest;
    }

    private SignatureConfiguration getPadesSignatureConfiguration() {

        Document document = new Document();
        document.setBytesB64(pdfB64);
        document.setName(PDF_NAME);

        Stamper stamper = new Stamper();
        stamper.setPage(1);
        stamper.setxAxis(80);
        stamper.setyAxis(700);
        stamper.setLogoB64(logoB64);
        stamper.setType(Type.QR_BARCODE128);
        stamper.setCsvPath(fortressDemoConfiguration.getApi().getUrl() + "v#");
        stamper.setRotation(null);

        PadesConfiguration padesConfiguration = new PadesConfiguration();
        padesConfiguration.setStamper(stamper);

        SignatureConfiguration signatureConfiguration = new SignatureConfiguration();
        signatureConfiguration.setPadesConfiguration(padesConfiguration);
        signatureConfiguration.setSignatureType(SignatureType.PADES_LTA);
        signatureConfiguration.setSignatureAlgorithm(SignatureAlgorithm.RSA_SHA256);
        signatureConfiguration.setPackaging(Packaging.ENVELOPED);
        signatureConfiguration.setDocument(document);
        signatureConfiguration.setTsa(fortressDemoConfiguration.getTsa());

        return signatureConfiguration;
    }

    private SignatureRequest getXadesSignatureRequest(String userCode) {
        ArrayList<SignatureConfiguration> signatureConfigurations = new ArrayList<>();
        signatureConfigurations.add(getXadesSignatureConfiguration());

        SignatureRequest signatureRequest = new SignatureRequest();
        signatureRequest.setSignatureConfigurations(signatureConfigurations);
        signatureRequest.setUserCode(userCode);

        return signatureRequest;
    }

    private SignatureConfiguration getXadesSignatureConfiguration() {
        Document document = new Document();
        document.setBytesB64(xmlB64);
        document.setName(XML_NAME);

        XadesConfiguration xadesConfiguration = new XadesConfiguration();
        xadesConfiguration.setDssReferenceUri("#id_node2");

        SignatureConfiguration signatureConfiguration = new SignatureConfiguration();
        signatureConfiguration.setXadesConfiguration(xadesConfiguration);
        signatureConfiguration.setSignatureType(SignatureType.XADES_T);
        signatureConfiguration.setSignatureAlgorithm(SignatureAlgorithm.RSA_SHA256);
        signatureConfiguration.setPackaging(Packaging.ENVELOPED);
        signatureConfiguration.setTsa(fortressDemoConfiguration.getTsa());
        signatureConfiguration.setDocument(document);

        return signatureConfiguration;
    }

    private SignatureRequest geBatchSignatureRequest(String userCode) {
        ArrayList<SignatureConfiguration> signatureConfigurations = new ArrayList<>();
        signatureConfigurations.add(getPadesSignatureConfiguration());
        signatureConfigurations.add(getXadesSignatureConfiguration());

        SignatureRequest signatureRequest = new SignatureRequest();
        signatureRequest.setSignatureConfigurations(signatureConfigurations);
        signatureRequest.setUserCode(userCode);
        return signatureRequest;
    }

    public AccessToken getAccessToken(String code, String redirectUri) throws FortressDemoException {
        try {
            return fortressApi.getAccessToken(code, redirectUri, GRANT_TYPE_AUTHORIZATION_CODE);
        } catch (ApiException e) {
            throw new FortressDemoException(e.getMessage(), e);
        }
    }

    public AccessToken getClientAccessToken() throws FortressDemoException {
        try {
            return fortressApi.getAccessToken("", redirectUri, GRANT_TYPE_CLIENT_CREDENTIALS);
        } catch (ApiException e) {
            throw new FortressDemoException(e.getMessage(), e);
        }
    }

    public String getAuthUri(String redirectUri, String userCode, String scope, String state) throws FortressDemoException {
        try {
            return fortressApi.getAuthorizationUri(redirectUri, userCode, scope, state);
        } catch (ApiException e) {
            throw new FortressDemoException(e.getMessage(), e);
        }
    }

    public String getAuthSignatureUri(String signatureCode) throws FortressDemoException {
        try {
            return fortressApi.getSignatureAuthorizationUri(signatureCode, redirectUri);
        } catch (ApiException e) {
            throw new FortressDemoException(e.getMessage(), e);
        }
    }

    public byte[] getDocument(String signatureCode) throws FortressDemoException {
        try {
            AccessToken accessToken = getClientAccessToken();
            return Base64.decodeBase64(fortressApi.getDocument(accessToken.getAccessToken(), signatureCode).getBytesB64().getBytes());
        } catch (ApiException e) {
            throw new FortressDemoException(e.getMessage(), e);
        }
    }

}
